#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: pharveycollard
"""

#%% Import packages
import numpy as np
import matplotlib.pyplot as plt, matplotlib
import scipy.io as sio
import os

import qcodes as qc
import qcodes.data.data_set as data_set

#%% Functions on quantum operators and states
def tensor(*args):
    """ Tensor product of operators or states to expand Hilbert space. 
    Example: $(\hat A) \otimes (\hat B)$, $|a>\otimes|b>$. """
    if len(args) > 1:
        op = np.kron(args[0], tensor(*args[1:]))
    else:
        op = args[0]
    return op

def cdot(*args):
    """ Scalar product of vectors of operators and/or scalars. 
    Example: $(\vec B) \cdot (\vec \hat \sigma)$."""
    if len(args) > 1:
        op = np.einsum('i...,i...', args[0], cdot(*args[1:]))
    else:
        op = args[0]
    return op

def ketbra(ket, bra):
    """ Make operator (ket-bra outer product) from two (non conjugated non 
    transposed) states.
    Vectorized call: multiple kets stored as row (axis=-1) vectors.
    Example: $|ket><bra|$. """
    # TEST: H1 = sigma_y*1.; H2 = diagonal_eigenstate_sum(*np.linalg.eigh(H1)); H1-H2
    # kb = np.outer(ket, bra.conj()) # non-vectorized
    kb = np.einsum('...i,...j', ket, bra.conj()) # vectorized 
    return kb

def braket(bra, ket):
    """ Bra-ket inner product from two (non conjugated non 
    transposed) states. 
    Vectorized call: multiple kets stored as row (axis=-1) vectors.
    Example: $<bra|ket>$. """
    # bk = np.inner(bra.conj(), ket) # non-vectorized
    bk = np.einsum('...i,...i', bra.conj(), ket) # vectorized 
    return bk

def braketsq(bra, ket):
    """ Probability or state overlap.
    Example: $|<bra|ket>|^2$. """
    prob = np.abs(braket(bra, ket))**2
    return prob

def make_a_dag(n):
    """ Creation operator for harmonic oscillator of Hilbert space 
    dimension n. """
    adag = 0*np.identity(n)
    for i in range(n-1):
        adag[i+1,i] = np.sqrt(i+1)
    return adag

def diagonal_eigenstate_sum(energies, eigenstates, indices=None):
    """
    Computes the Hamiltonain operator H = \sum_i E_i |i><i|, where i can 
    be restained to a subset of indices. dot(H, v[:, i]) = E[i] * v[:, i]

    Parameters
    ----------
    energies : ndarray, vector
        As provided by the numpy.linalg.eigh() function.
    eigenstates : ndarray, matrix
        As provided by the numpy.linalg.eigh() function.
    indices : List or None, optional
        Eigenstates to include. The default None means all.

    Returns
    -------
    H : ndarray, square matrix
        The Hamiltonian H.

    """
    if indices == None:
        indices = range(energies.shape[0]) # all indices
    H = 0 * ketbra(eigenstates[:,0], eigenstates[:,0]) # initialize H
    for i in indices:
        H = H + energies[i] * ketbra(eigenstates[:,i], eigenstates[:,i])
    return H

def select_eigenstates(energies, eigenstates, indices=None):
    """
    Selects a subset of the eigenstates. dot(H, v[:, i]) = E[i] * v[:, i]

    Parameters
    ----------
    energies : ndarray, vector
        As provided by the numpy.linalg.eigh() function.
    eigenstates : ndarray, matrix
        As provided by the numpy.linalg.eigh() function.
    indices : List or None, optional
        Eigenstates to include. The default None means all.

    Returns
    -------
    states : ndarray, matrix
        The selected states.

    """
    if indices == None:
        indices = range(energies.shape[0])
    states = eigenstates[:,indices]
    return states

def cart2sphd(x, y, z):
    """ Converts vectors of 3D cartesian coordinates (x, y, z) to spherical 
    coordinates (r, theta, phi) in degrees. Convention: Wikipedia 'math', 
    https://en.wikipedia.org/wiki/Spherical_coordinate_system """
    rho = np.hypot(x, y)
    r = np.hypot(rho, z)
    theta = np.arctan2(y, x)
    phi = np.arctan2(rho, z)
    theta = theta/np.pi*180
    phi = phi/np.pi*180
    return np.array([r, theta, phi])

def sph2cartd(r, theta, phi):
    """ Converts vectors of 3D spherical coordinates (r, theta, phi) in degrees 
    to cartesian coordinates (x, y, z). Convention: Wikipedia 'math', 
    https://en.wikipedia.org/wiki/Spherical_coordinate_system """
    theta = theta/180*np.pi
    phi = phi/180*np.pi
    x = r * np.sin(phi) * np.cos(theta)
    y = r * np.sin(phi) * np.sin(theta)
    z = r * np.cos(phi)
    return np.array([x, y, z])

#%% Functions to find eigenvalues and transition energies
NON_VECT_PARAMS = ( # Non-vectorizable parameters in the Hamiltonian
    'DQD1_EIGV_TO_INCL',
    'DQD2_EIGV_TO_INCL',
    'RES_EIGV_TO_INCL',
)

def calc_energies(H_fun, **kwargs):
    """
    Calculates the eigenenergies of a Hamiltonian.

    Parameters
    ----------
    H_fun : Function.
        The function (without arguments) that implements the Hamiltonian.
    **kwargs : key=value pairs.
        Arguments to be passed on to the Hamiltonian function. Only one can be 
        a vector. The others need to be scalar, or in the NON_VECT_PARAMS list.

    Returns
    -------
    energies : [len(kwargs),len(eigenenergies)] ndarray
        The eigenenergies for each value of the -vector- argument.

    """
    def select_energies(H_fun, **kwargs):
        """ This kwargs is not vectorized. """
        H, _ = H_fun(**kwargs)
        ee = np.linalg.eigvalsh(H)
        return ee
    
    vector_key = ''
    vector_len = 1
    for key in kwargs:
        if (np.size(kwargs[key]) > 1) & (key not in NON_VECT_PARAMS):
            vector_key = key
            vector_len = np.size(kwargs[key])
            for i in range(vector_len):
                kwargs2 = kwargs.copy()
                kwargs2[vector_key] = kwargs[vector_key][i]
                ee = select_energies(H_fun, **kwargs2)
                if i == 0:
                    energies = ee
                else:
                    energies = np.vstack((energies, ee))
            break # do not go through the remaining items
    if vector_len == 1:
        energies = select_energies(H_fun, **kwargs)
    return energies

def calc_energies_filter(H_fun, **kwargs):
    """
    Calculates the eigenenergies of a Hamiltonian. Returns the energies 
    filtered by state.

    Parameters
    ----------
    H_fun : Function.
        The function (without arguments) that implements the Hamiltonian.
    **kwargs : key=value pairs.
        Arguments to be passed on to the Hamiltonian function. Only one can be 
        a vector. The others need to be scalar, or in the NON_VECT_PARAMS list.

    Returns
    -------
    energies : [len(kwargs),len(eigenenergies)] ndarray
        The eigenenergies for each value of the -vector- argument.

    """
    def select_energies(H_fun, **kwargs):
        """ This kwargs is not vectorized. """
        H, states_to_incl = H_fun(**kwargs)
        ee, vv = np.linalg.eigh(H)
        energies = np.zeros(len(states_to_incl))
        for i, state in enumerate(states_to_incl):
            overlaps = braketsq(vv.T, state)
            idx = np.argmax(overlaps)
            energies[i] = ee[idx]
            ee = np.delete(ee, idx, axis=0)
            vv = np.delete(vv, idx, axis=1)
        return energies
    
    vector_key = ''
    vector_len = 1
    for key in kwargs:
        if (np.size(kwargs[key]) > 1) & (key not in NON_VECT_PARAMS):
            vector_key = key
            vector_len = np.size(kwargs[key])
            for i in range(vector_len):
                kwargs2 = kwargs.copy()
                kwargs2[vector_key] = kwargs[vector_key][i]
                ee = select_energies(H_fun, **kwargs2)
                if i == 0:
                    energies = ee
                else:
                    energies = np.vstack((energies, ee))
            break # do not go through the remaining items
    if vector_len == 1:
        energies = select_energies(H_fun, **kwargs)
    return energies

def comb(n, k):
    """ Compute P_k^n, the number of distinct non-ordered sets of k elements 
    from a set of n elements. """
    c = np.math.factorial(n) // np.math.factorial(k) // np.math.factorial(n-k)
    return c

def calc_transitions(energies):
    """ Calculates all the possible energy differences. """
    # The number of transitions should be comb(n, 2)
    # OPTIMIZATION: Use itertools.permutations from itertools module?
    if np.ndim(energies) == 1:
        energies = np.array([energies])
    transitions = [[]]
    if np.size(energies, 1) > 1:
        new_t = calc_transitions(energies[:,1:])
        if np.size(new_t) > 0:
            transitions = np.hstack((new_t, energies[:,1:]-energies[:,0,None]))
        else:
            transitions = energies[:,1:]-energies[:,0,None]
    return np.abs(transitions)

def calc_t_f(H_fun, **kwargs):
    transitions = calc_transitions( calc_energies_filter(H_fun, **kwargs) )
    return transitions

def mod_param(params, *arg_pairs, **kwargs):
    """ Returns a copy of params with a modified key:new_value. Useful for plots. 
    Optional *arg_pairs are aditionnal (key, new_value) pairs in sequence.
    Optional **kwargs are aditionnal key=value keywords. """
    new_params = params.copy()
    for k, v in zip(arg_pairs[0::2], arg_pairs[1::2]):
        new_params[k] = v
    for k, v in kwargs.items():
        new_params[k] = v
    return new_params

def move_figure(fh, x, y):
    import matplotlib
    """ Move figure's upper left corner to pixel (x, y). """
    backend = matplotlib.get_backend()
    if backend == 'TkAgg':
        fh.canvas.manager.window.wm_geometry("+%d+%d" % (x, y))
    elif backend == 'WXAgg':
        fh.canvas.manager.window.SetPosition((x, y))
    else:
        # This works for QT and GTK
        # You can also use window.setGeometry
        fh.canvas.manager.window.move(x, y)

def disc(x, y, threshold=None):
    """ Remove discontinuities in plot by replacing with NaNs when jumps are 
    larger than a certain threshold. (conserves number of points, but removes 
    data) """
    if threshold == None:
        # threshold = np.percentile(np.abs(np.diff(y)), 99)
        threshold = 1e-2
    y = np.where(np.abs(np.diff(y, prepend=y[0])) <= threshold, y, np.full(y.shape, np.nan))
    return x, y

def disc2(x, y, threshold=None):
    """ Remove discontinuities in plot by inserting NaNs when jumps are larger 
    than a certain threshold. (preferred method, but expands nb of pts) """
    if threshold == None:
        # threshold = np.percentile(np.abs(np.diff(y)), 99)
        # threshold = 3*np.std(np.abs(np.diff(y)))
        # threshold = (np.max(y) - np.min(y)) / 50
        threshold = 1e-2
    pos = np.where(np.abs(np.diff(y)) >= threshold)[0] + 1
    x = np.insert(x.astype(np.double), pos, np.nan)
    y = np.insert(y.astype(np.double), pos, np.nan)
    return x, y

#%% Generate Hamiltonian

# Constants
planckh = 4.136/1e9 # ueV/Hz
muB = 57.88 # ueV/T
gL = 2 # electron g-factor

# Charge and spin operators
idm = np.identity(2)
sigma_x = np.array([[0,1],[1,0]])
sigma_y = np.array([[0,-1j],[1j,0]])
sigma_z = np.array([[1,0],[0,-1]])
s0 = np.array([0,1]) # sigma_z base vector

sigma_p = sigma_x + 1j*sigma_y # spin ladder operator
sigma_m = sigma_x - 1j*sigma_y # spin ladder operator
sigma_vct = np.array([sigma_x, sigma_y, sigma_z]) # spin vector operator

# Harmonic operators
dim_r = 5 # min 3 for correct chi_c, or 5 for chi_s

a_dag = make_a_dag(dim_r)
a = a_dag.conj().T
idm_r = np.identity(dim_r)
a0 = np.array([1,0,0]) # a^\dagger.a base vector

def um_field(uvct, B_um_offset, chi_um=0, B_ext=(0,0,0), B_ext_offset=(0,0,0)):
    """ Returns the micromagnet field in cartesian coordinates. 
    
    Parameters
    ----------
    uvct : np.ndarray, or tuple or list that can be converted to a numpy array
        Unit vector pointing at micromagnet field, in cartesian coordinates.
    B_um_offset : scalar
        Micromagnet field.
    chi_um : scalar
        Micromagnet susceptibility to external field.
    B_ext : np.ndarray, or tuple or list that can be converted to a numpy array
        External magnetic field.
    B_ext_offset : np.ndarray, or tuple or list that can be converted to a 
    numpy array (optionnal)
    
    Returns
    -------
    B_um : np.ndarray
    
    """
    # Convert/enforce vectors to ndarray type
    uvct = np.array(uvct)
    B_ext = np.array(B_ext)
    B_ext_offset = np.array(B_ext_offset)
    # Calculate um field
    B_um = (B_um_offset + chi_um*cdot(B_ext-B_ext_offset, uvct)) * uvct
    return B_um

def H_system(eps1, t_c1, B_um1, DB_um1, g_c1, chi_um1, 
             eps2, t_c2, B_um2, DB_um2, g_c2, chi_um2, 
             w_r, Br, Bphi, Br_off,
             DQD1_EIGV_TO_INCL, DQD2_EIGV_TO_INCL, RES_EIGV_TO_INCL):
    """ 
    Two DQDs with one spin each coupled to a resonator.
    Hamiltonian inspired from [M. Benito ... Phys. Rev. B 96, 235434 (2017)].
    """
    bext = sph2cartd(Br, -90, Bphi)
    bextoff = sph2cartd(Br_off, -90, Bphi)
    um1 = sph2cartd(1, -90+180, 15) # um unit vector, face-up is -x
    um2 = sph2cartd(1, +90+180, 15) # um unit vector, face-up is -x
    # Convert B fields in spherical [T] to h in cartesian [Hz]
    hext = (gL*muB/planckh) * bext
    hum1 = (gL*muB/planckh) * um_field(um1, B_um1, chi_um1, bext, bextoff)
    dhum1 = (gL*muB/planckh) * sph2cartd(DB_um1, 0+180, 90) # face-up is -x
    hum2 = (gL*muB/planckh) * um_field(um2, B_um2, chi_um2, bext, bextoff)
    dhum2 = (gL*muB/planckh) * sph2cartd(DB_um2, 0+180, 90) # face-up is -x
    # Calculate average (h) and difference (dh) cartesian vector fields in [Hz]
    h1 = hext + hum1 # average dqd1 field
    dh1 = dhum1 # difference dqd1 field
    h2 = hext + hum2 # average dqd2 field
    dh2 = dhum2 # difference dqd2 field
    
    # charge1 x spin1
    # units: Hz
    def Hdqd(eps, t_c, h, dh):
        H = 1/2*(eps * tensor(sigma_z, idm)
                 + 2*t_c * tensor(sigma_x, idm)
                 + tensor(idm, cdot(h, sigma_vct))
                 + tensor(sigma_z, cdot(dh/2, sigma_vct)))
        return H

    # charge1 x spin1 x charge2 x spin2 x photon
    # units: Hz
    H1 = Hdqd(eps1, t_c1, h1, dh1)
    states1 = select_eigenstates(*np.linalg.eigh(H1), DQD1_EIGV_TO_INCL)
    # H1 = diagonal_eigenstate_sum(*np.linalg.eigh(H1), DQD1_EIGV_TO_INCL) # Consistency check
    
    H2 = Hdqd(eps2, t_c2, h2, dh2)
    states2 = select_eigenstates(*np.linalg.eigh(H2), DQD2_EIGV_TO_INCL)
    # H2 = diagonal_eigenstate_sum(*np.linalg.eigh(H2), DQD2_EIGV_TO_INCL) # Consistency check
    
    H1 = tensor(H1, idm, idm, idm_r) # expand to full Hilbert space
    H2 = tensor(idm, idm, H2, idm_r) # expand to full Hilbert space
    
    Hr = w_r * tensor(idm, idm, idm, idm, a_dag.dot(a)) 
    Hi1 = g_c1 * tensor(sigma_z, idm, idm, idm, a+a_dag)
    Hi2 = g_c2 * tensor(idm, idm, sigma_z, idm, a+a_dag)
    
    # List states to include
    states_to_incl = []
    for i in range(np.size(states1, 1)):
        for j in range(np.size(states2, 1)):
            for k in RES_EIGV_TO_INCL:
                state = np.zeros(dim_r)
                state[k] = 1
                state = tensor(states1[:,i], states2[:,j], state) # expand to full Hilbert space
                states_to_incl.append(state)
    
    return (H1 + H2 + Hr + Hi1 + Hi2), states_to_incl

dim_tot = 2*2*2*2*dim_r

#%% Computed parameters
def compute_disp_shift(H_system, params, print_vals=True):
    """ Compute charge qubit dispersive shifts chi_c numerically. 
    (w_r' = w_r - chi) """
    params_disp = mod_param(params,
                            'eps1', 0e9, 'eps2', 0e9,
                            'Br', 0.1, 'Bphi', 0,
                            'B_um1', 0, 'B_um2', 0,
                            'DB_um1', 0, 'DB_um2', 0,
                            'chi_um1', 0, 'chi_um2', 0,
                            'DQD1_EIGV_TO_INCL', [0, ],
                            'DQD2_EIGV_TO_INCL', [0, ],
                            'RES_EIGV_TO_INCL', [0, 1])
    
    y_energy = calc_energies_filter(H_system, **mod_param(params_disp, 'eps2', 1000e9))
    y_transitions = calc_transitions(y_energy)
    disp_shift1 = params['w_r'] - y_transitions[0,0]
    
    y_energy = calc_energies_filter(H_system, **mod_param(params_disp, 'eps1', 1000e9))
    y_transitions = calc_transitions(y_energy)
    disp_shift2 = params['w_r'] - y_transitions[0,0]
    
    y_energy = calc_energies_filter(H_system, **params_disp)
    y_transitions = calc_transitions(y_energy)
    disp_shift = params['w_r'] - y_transitions[0,0]
    
    if print_vals == True:
        print('Disp shift DQD1 = {:.4g} MHz'.format(disp_shift1/1e6))
        print('Disp shift DQD2 = {:.4g} MHz'.format(disp_shift2/1e6))
        print('Disp shift DQD1+2 = {:.4g} MHz'.format(disp_shift/1e6))
        print('f_disp DQD1 = {:.5g} GHz'.format(params['w_r']/1e9-disp_shift1/1e9))
        print('f_disp DQD2 = {:.5g} GHz'.format(params['w_r']/1e9-disp_shift2/1e9))
        print('f_disp DQD1+2 = {:.5g} GHz'.format(params['w_r']/1e9-disp_shift/1e9))
    
    return disp_shift1, disp_shift2, disp_shift

def compute_SO_shift(H_system, params, print_vals=True):
    """ Compute spin-orbit qubit shifts at zero detuning chi_SO numerically. 
    (w_s' = w_s + chi) """
    # DQD1
    params_chiSO = mod_param(params,
                            'eps1', 1000e9, 'eps2', 1000e9,
                            'g_c1', 0, 'g_c2', 0,
                            'DQD1_EIGV_TO_INCL', [0, 1],
                            'DQD2_EIGV_TO_INCL', [0, ],
                            'RES_EIGV_TO_INCL', [0, ])
    y_energy = calc_energies_filter(H_system, **mod_param(params_chiSO, 'eps1', 1000e9))
    y_transitions = calc_transitions(y_energy)
    s1 = y_transitions[0,0]
    y_energy = calc_energies_filter(H_system, **mod_param(params_chiSO, 'eps1', 0e9))
    y_transitions = calc_transitions(y_energy)
    s0 = y_transitions[0,0]
    SO_shift1 = s0 - s1
    
    # DQD2
    params_chiSO = mod_param(params,
                            'eps1', 1000e9, 'eps2', 1000e9,
                            'g_c1', 0, 'g_c2', 0,
                            'DQD1_EIGV_TO_INCL', [0, ],
                            'DQD2_EIGV_TO_INCL', [0, 1],
                            'RES_EIGV_TO_INCL', [0, ])
    y_energy = calc_energies_filter(H_system, **mod_param(params_chiSO, 'eps2', 1000e9))
    y_transitions = calc_transitions(y_energy)
    s1 = y_transitions[0,0]
    y_energy = calc_energies_filter(H_system, **mod_param(params_chiSO, 'eps2', 0e9))
    y_transitions = calc_transitions(y_energy)
    s0 = y_transitions[0,0]
    SO_shift2 = s0 - s1
    
    if print_vals == True:
        print('SO shift DQD1 = {:.4g} MHz'.format(SO_shift1/1e6))
        print('SO shift DQD2 = {:.4g} MHz'.format(SO_shift2/1e6))
    
    return SO_shift1, SO_shift2

def compute_gs(H_system, params, print_vals=True):
    """ Compute g_s and Br at resonance numerically. 
    (returns g_s > 0) """
    # DQD1
    x_Br = np.arange(53e-3, 54e-3, 0.01e-3) # make this precise for accurate values
    B_res1 = (params['w_r']-disp_shift1)/(gL*muB/planckh) # exact if disp_shift is exact
    params_b = mod_param(params,
                         'eps1', 0e9, 'eps2', 1000e9,
                         'Br', x_Br,
                         'DQD1_EIGV_TO_INCL', [0, 1],
                         'DQD2_EIGV_TO_INCL', [0, ],
                         'RES_EIGV_TO_INCL', [0, ])
    y_energy = calc_energies_filter(H_system, **params_b)
    y_transitions = calc_transitions(y_energy)
    idx = np.argmin(np.abs(y_transitions[:,0]/(gL*muB/planckh) - B_res1))
    Br_res1 = x_Br[idx]
    g_s1 = np.max(np.diff(y_transitions[:,0]))/2 # find the discontinuity in the curve (approx.)
    
    # DQD2
    B_res2 = (params['w_r']-disp_shift2)/(gL*muB/planckh) # exact if disp_shift is exact
    params_b = mod_param(params,
                         'eps1', 1000e9, 'eps2', 0e9,
                         'Br', x_Br,
                         'DQD1_EIGV_TO_INCL', [0, ],
                         'DQD2_EIGV_TO_INCL', [0, 1],
                         'RES_EIGV_TO_INCL', [0, ])
    y_energy = calc_energies_filter(H_system, **params_b)
    y_transitions = calc_transitions(y_energy)
    idx = np.argmin(np.abs(y_transitions[:,0]/(gL*muB/planckh) - B_res2))
    Br_res2 = x_Br[idx]
    g_s2 = np.max(np.diff(y_transitions[:,0]))/2 # find the discontinuity in the curve (approx.)
    
    if print_vals == True:
        print('Br at resonance DQD1 = {:.2f} mT'.format(Br_res1/1e-3))
        print('Br at resonance DQD2 = {:.2f} mT'.format(Br_res2/1e-3))
        print('g_s1/2π = {:.3g} MHz'.format(g_s1/1e6))
        print('g_s2/2π = {:.3g} MHz'.format(g_s2/1e6))
    
    # to plot the transition used
    # plt.figure(); plt.plot(x_Br, y_transitions[:,0]/(gL*muB/planckh)) 
    
    return g_s1, g_s2, Br_res1, Br_res2

def compute_delta_chi(H_system, params, print_vals=True):
    """ Compute spin qubit delta and chi numerically. delta is 'bare'.
    Normally, delta_d-delta_b=2*chi, but charge d.o.f. skews this a bit.
    (w_q' = w_q + chi) and (delta = w_q - w_r) """
    # dressed resonator frequencies
    params_disp = mod_param(params,
                            'eps1', 0e9, 'eps2', 0e9,
                            'DQD1_EIGV_TO_INCL', [0, ],
                            'DQD2_EIGV_TO_INCL', [0, ],
                            'RES_EIGV_TO_INCL', [0, 1])
    y_energy = calc_energies_filter(H_system, **mod_param(params_disp, 'eps2', 1000e9))
    y_transitions = calc_transitions(y_energy)
    r1 = y_transitions[0,0]
    y_energy = calc_energies_filter(H_system, **mod_param(params_disp, 'eps1', 1000e9))
    y_transitions = calc_transitions(y_energy)
    r2 = y_transitions[0,0]
    # y_energy = calc_energies_filter(H_system, **params_disp)
    # y_transitions = calc_transitions(y_energy)
    # r12 = y_transitions[0,0]
    
    # DQD1, chi
    params_chi = mod_param(params,
                           'eps1', 0e9, 'eps2', 1000e9,
                           'DQD1_EIGV_TO_INCL', [0, 1],
                           'DQD2_EIGV_TO_INCL', [0, ],
                           'RES_EIGV_TO_INCL', [0, ])
    y_energy = calc_energies_filter(H_system, **params_chi)
    y_transitions = calc_transitions(y_energy)
    e0 = y_transitions[0,0]
    y_energy = calc_energies_filter(H_system, **mod_param(params_chi, 'RES_EIGV_TO_INCL', [1, ]))
    y_transitions = calc_transitions(y_energy)
    e1 = y_transitions[0,0]
    chi1 = (e1 - e0)/2
    delta_d1 = e0 - r1 # dressed delta
    # DQD1, delta
    y_energy = calc_energies_filter(H_system, **mod_param(params_chi, 'g_c1', 0))
    y_transitions = calc_transitions(y_energy)
    e0 = y_transitions[0,0]
    delta_b1 = e0 - (params['w_r']-disp_shift1) # disp_shift is global, not clean
    
    # DQD2, chi
    params_chi = mod_param(params,
                           'eps1', 1000e9, 'eps2', 0e9,
                           'DQD1_EIGV_TO_INCL', [0, ],
                           'DQD2_EIGV_TO_INCL', [0, 1],
                           'RES_EIGV_TO_INCL', [0, ])
    y_energy = calc_energies_filter(H_system, **params_chi)
    y_transitions = calc_transitions(y_energy)
    e0 = y_transitions[0,0]
    y_energy = calc_energies_filter(H_system, **mod_param(params_chi, 'RES_EIGV_TO_INCL', [1, ]))
    y_transitions = calc_transitions(y_energy)
    e1 = y_transitions[0,0]
    chi2 = (e1 - e0)/2
    delta_d2 = e0 - r2 # dressed delta
    # DQD2, delta
    y_energy = calc_energies_filter(H_system, **mod_param(params_chi, 'g_c2', 0))
    y_transitions = calc_transitions(y_energy)
    e0 = y_transitions[0,0]
    delta_b2 = e0 - (params['w_r']-disp_shift2) # disp_shift is global, not clean
    
    if print_vals == True:
        print('χ_1/2π = {:.3g} MHz'.format(chi1/1e6))
        print('χ_2/2π = {:.3g} MHz'.format(chi2/1e6))
        print('∆_1/2π = {:.4g} MHz (bare)'.format(delta_b1/1e6))
        print('∆_2/2π = {:.4g} MHz (bare)'.format(delta_b2/1e6))
        print('∆_1/2π = {:.4g} MHz (dressed)'.format(delta_d1/1e6))
        print('∆_2/2π = {:.4g} MHz (dressed)'.format(delta_d2/1e6))
    
    return delta_b1, delta_b2, chi1, chi2

def compute_J(H_system, params, print_vals=True):
    """ Compute spin-spin dispersive interaction J_s numerically. """
    x_Bphi = np.arange(9.5, 11.5, 0.05) # make this precise for accurate values
    params_J = mod_param(params,
                           'eps1', 0e9, 'eps2', 0e9,
                           'Bphi', x_Bphi,
                           'DQD1_EIGV_TO_INCL', [0, ],
                           'DQD2_EIGV_TO_INCL', [0, 1],
                           'RES_EIGV_TO_INCL', [0, ])
    y_energy = calc_energies_filter(H_system, **params_J)
    y_transitions = calc_transitions(y_energy)
    diffs = (np.abs(np.diff(y_transitions[:,0])))/2 # find the discontinuity in the curve (approx.)
    i = np.argmax(diffs) # find the discontinuity in the curve (approx.)
    J_s = diffs[i]
    Bphi_Js = x_Bphi[i]
    
    if print_vals == True:
        print('J_s/2π = {:.3g} MHz, Bphi = {:.4g} deg'.format(J_s/1e6, Bphi_Js))
    
    # to plot the transition used
    # plt.figure(); plt.plot(x_Bphi, y_transitions[:,0]/1e9,'.') 
    
    return J_s, Bphi_Js

def compute_delta_Js(H_system, params, Bphi_Js, print_vals=True):
    """ Compute spin qubit bare delta at spin-spin resonance numerically. """
    
    
    # bare spin frequency 1
    params_delta = mod_param(params,
                           'eps1', 0e9, 'eps2', 1000e9,
                           'Bphi', Bphi_Js,
                           'g_c1', 0, 'g_c2', 0,
                           'DQD1_EIGV_TO_INCL', [0, 1],
                           'DQD2_EIGV_TO_INCL', [0, ],
                           'RES_EIGV_TO_INCL', [0, ])
    y_energy = calc_energies_filter(H_system, **params_delta)
    y_transitions = calc_transitions(y_energy)
    s1 = y_transitions[0,0]
    delta1 = s1 - (params['w_r']-disp_shift) # disp_shift is global, not clean
    
    # bare spin frequency 2
    params_delta = mod_param(params,
                           'eps1', 1000e9, 'eps2', 0e9,
                           'Bphi', Bphi_Js,
                           'g_c1', 0, 'g_c2', 0,
                           'DQD1_EIGV_TO_INCL', [0, ],
                           'DQD2_EIGV_TO_INCL', [0, 1],
                           'RES_EIGV_TO_INCL', [0, ])
    y_energy = calc_energies_filter(H_system, **params_delta)
    y_transitions = calc_transitions(y_energy)
    s2 = y_transitions[0,0]
    delta2 = s2 - (params['w_r']-disp_shift) # disp_shift is global, not clean
    
    delta_Js = (delta1+delta2)/2
    
    if print_vals == True:
        print('∆_Js/2π = {:.4g} MHz (bare, spin1 = spin2)'.format(delta_Js/1e6))
    
    return delta1, delta2

#%% Base parameters
#import pdb; pdb.set_trace()
params = {'eps1': 0e9,
          'eps2': 0e9,
          't_c1': 13.2e9/2,
          't_c2': 13.7e9/2,
          'Br': 52e-3,
          'Bphi': 10.5,
          'B_um1': 0.1650 - 20e-3*0.67*np.cos((15+10.2)/180*np.pi) +0.0003,
          'B_um2': 0.1600 - 20e-3*0.63*np.cos((-15+10.2)/180*np.pi) -0.0001,
          'DB_um1': 42e-3,
          'DB_um2': 42e-3,
          'chi_um1': 0.67,
          'chi_um2': 0.63,
          'Br_off': -20e-3,
          'w_r': 6.916e9, 
          'g_c1': 192e6,
          'g_c2': -192e6,
          'DQD1_EIGV_TO_INCL': [0, 1],
          'DQD2_EIGV_TO_INCL': [0, 1],
          'RES_EIGV_TO_INCL': [0, 1]}

# Analytical disp shift incl Bloch-Siegert (approx.)
chic_disp_calc1 = params['g_c1']**2*(2*params['t_c1']-params['w_r'])/(2*params['t_c1']-params['w_r'])**2
chic_BS_calc1 = params['g_c1']**2*(2*params['t_c1']+params['w_r'])/(2*params['t_c1']+params['w_r'])**2
disp_shift_calc1 = chic_disp_calc1 + chic_BS_calc1
chic_disp_calc2 = params['g_c2']**2*(2*params['t_c2']-params['w_r'])/(2*params['t_c2']-params['w_r'])**2
chic_BS_calc2 = params['g_c2']**2*(2*params['t_c2']+params['w_r'])/(2*params['t_c2']+params['w_r'])**2
disp_shift_calc2 = chic_disp_calc2 + chic_BS_calc2
disp_shift_calc = disp_shift_calc1 + disp_shift_calc2
# print('Disp shift DQD1 (calc) = {:.4g} MHz'.format(disp_shift_calc1/1e6))
# print('Disp shift DQD2 (calc) = {:.4g} MHz'.format(disp_shift_calc2/1e6))
# print('f_disp DQD1 (calc) = {:.5g} GHz'.format(params['w_r']/1e9-disp_shift_calc1/1e9))
# print('f_disp DQD2 (calc) = {:.5g} GHz'.format(params['w_r']/1e9-disp_shift_calc2/1e9))

#% Compute dispersive shifts numerically
disp_shift1, disp_shift2, disp_shift = compute_disp_shift(H_system, params)

#% Compute SO shifts numerically
SO_shift1, SO_shift2 = compute_SO_shift(H_system, params)

#% Compute g_s numerically
#% Compute Br at spin-photon resonance
g_s1, g_s2, Br_res1, Br_res2 = compute_gs(H_system, params)

#% Compute delta and chi numerically
delta1, delta2, chi1, chi2 = compute_delta_chi(H_system, params)

# Compute dispersive J
J_s, Bphi_Js = compute_J(H_system, params)
delta1_Js, delta2_Js = compute_delta_Js(H_system, params, Bphi_Js)

# pprint(params)

#%% Plot vs detuning
# x_eps = np.linspace(-100e9, 100e9, 1001)
# y_energy = calc_energies(H_system, **mod_param(params, 'eps1', x_eps))

# # plt.close(1)
# # plt.figure(1)
# # for i in range(0, np.size(y_energy,1)):
# #     plt.plot(x_eps/1e9, y_energy[:,i]/1e9)
# # plt.xlabel('Epsilon (GHz)')
# # plt.ylabel('Energy (GHz)')
# # plt.show()

# #% Plot transition energies
# y_transitions = calc_transitions(y_energy[:,0:6])
# # y_transitions = calc_transitions(y_energy)

# plt.close(2)
# plt.figure(2)
# for i in range(0, np.size(y_transitions,1)):
#     plt.plot(x_eps/1e9, y_transitions[:,i]/1e9)
# plt.xlabel('Epsilon (GHz)')
# plt.ylabel('Transition energy (GHz)')
# plt.show()


# # plt.xlim(-5*params['t_c1']/1e9, 5*params['t_c1']/1e9)
# plt.ylim(params['w_r']/1e9-.02, params['w_r']/1e9+.02)

# # plt.ylim(-.1, 20)

# # plt.ylim(params['w_r']/1e9-.010-.025/2, params['w_r']/1e9-.010+.025/2)

#%% Plot vs Br
# x_Br = np.linspace(46e-3, 60e-3, 301)
# y_energy = calc_energies(H_system, **mod_param(params, 'Br', x_Br))

# # plt.close(3)
# # plt.figure(3)
# # for i in range(0, np.size(y_energy,1)):
    
# #     plt.plot(x_Br, y_energy[:,i]/1e9)
# # plt.xlabel('Br (T)')
# # plt.ylabel('Energy (GHz)')
# # plt.show()

# #% Plot transition energies
# y_transitions = calc_transitions(y_energy[:,0:6])
# # y_transitions = calc_transitions(y_energy)

# plt.close(4)
# plt.figure(4)
# for i in range(0, np.size(y_transitions,1)):
#     plt.plot(x_Br, y_transitions[:,i]/1e9)
# plt.xlabel('Br (T)')
# plt.ylabel('Transition energy (GHz)')
# plt.show()


# # plt.xlim(Br_res-.005, Br_res+.005*2)
# plt.ylim(params['w_r']/1e9-disp_shift/1e9-.080/2, params['w_r']/1e9-disp_shift/1e9+.080/2)

# # plt.ylim(-.1, 20)

#%% Plot vs Bphi
# x_Bphi = np.linspace(-15, 15, 301)
# y_energy = calc_energies(H_system, **mod_param(params, 'Bphi', x_Bphi))

# # plt.close(5)
# # plt.figure(5)
# # for i in range(0, np.size(y_energy,1)):
# #     plt.plot(x_Bphi, y_energy[:,i]/1e9)
# # plt.xlabel('Bphi (deg)')
# # plt.ylabel('Energy (GHz)')
# # plt.show()

# #% Plot transition energies
# y_transitions = calc_transitions(y_energy[:,0:6])
# # y_transitions = calc_transitions(y_energy)
# # y_transitions = calc_transitions(y_energy[:,0:14])

# plt.close(6)
# plt.figure(6)
# for i in range(0, np.size(y_transitions,1)):
#     plt.plot(x_Bphi, y_transitions[:,i]/1e9)
# plt.xlabel('Bphi (deg)')
# plt.ylabel('Transition energy (GHz)')
# plt.show()


# plt.xlim(8, 12)
# # plt.ylim(params['w_r']/1e9-2*disp_shift_calc/1e9-.080/2, params['w_r']/1e9-2*disp_shift_calc/1e9+.080/2+.060)
# plt.ylim(6.94, 7.02)
# # plt.ylim(6.94-.15, 7.02-.15)

# # plt.ylim(-.1, 20)

#%% Matplotlib rc config 
# plt.rcdefaults() # reset rc defaluts

FONT_SIZE = 7
TITLE_SIZE = 7
LW = 0.75
# set subscript font size
matplotlib.mathtext.SHRINK_FACTOR = 0.8

FIG_WIDTH = 5.5 # cm
FIG_HEIGHT = 5.5 # cm
fig_size_list = [FIG_WIDTH/2.51, FIG_HEIGHT/2.51] # in inches
ax_position_tup = (1.5/FIG_WIDTH, 1.0/FIG_HEIGHT, 3.5/FIG_WIDTH, 3.5/FIG_HEIGHT) # relative units

plt.rcParams['font.sans-serif'] = ['Helvetica Neue',
                                   'Helvetica',
                                   'DejaVu Sans',
                                   'Bitstream Vera Sans',
                                   'Computer Modern Sans Serif',
                                   'Lucida Grande',
                                   'Verdana',
                                   'Geneva',
                                   'Lucid',
                                   'Arial',
                                   'Helvetica',
                                   'Avant Garde',
                                   'sans-serif']

# plt.rcParams['mathtext.fontset'] = 'dejavusans'
# plt.rcParams['mathtext.fontset'] = 'stixsans'
plt.rcParams['mathtext.fontset'] = 'custom'
plt.rcParams['mathtext.it'] = 'Arial:italic' # closest to helvetica that works...
plt.rcParams['mathtext.rm'] = 'Helvetica Neue' # works

plt.rc('font', size=FONT_SIZE)          # controls default text sizes
plt.rc('axes', labelsize=FONT_SIZE)     # fontsize of the x and y labels
plt.rc('axes', titlesize=TITLE_SIZE)    # fontsize of the axes title
plt.rc('xtick', labelsize=FONT_SIZE)    # fontsize of the tick labels
plt.rc('ytick', labelsize=FONT_SIZE)    # fontsize of the tick labels
plt.rc('legend', fontsize=FONT_SIZE, title_fontsize=None)    # legend fontsize
plt.rc('figure', titlesize=TITLE_SIZE)  # fontsize of the figure title

plt.rc('axes', linewidth=LW)
plt.rc('lines', linewidth=LW)

TL = 2.5
plt.rc('xtick.major', size=TL)           # tick length (default 3.5)
plt.rc('xtick.minor', size=TL/1.75)      # tick length (default 2)
plt.rc('ytick.major', size=TL)           # tick length (default 3.5)
plt.rc('ytick.minor', size=TL/1.75)      # tick length (default 2)

pcolor_opts = {'shading': 'nearest',
               'rasterized': True, }  # pass as **pcolor_opts

plt.rcParams['legend.frameon'] = True

plt.rcParams['figure.dpi'] = 129.0 # adjust to match display
plt.rcParams['figure.figsize'] = fig_size_list # in inches

plt.rcParams['savefig.dpi'] = 300
plt.rcParams['savefig.transparent'] = True

plt.rcParams['pdf.fonttype'] = 3 # 42 (TrueType) to edit PDF ext editor. Default 3 is more compact.

def set_plot_area(ah):
    ah.set_position(ax_position_tup)  # relative units

def make_colorbar_top(ah, ph, label='', clims=(None, None)):
    """ Generates a colorbar at the outer top right corner of the plot. 
    
    Clims must be set in this function, otherwise the colorbar resets itself. Ex.:
    ch, _, _ = make_colorbar_top(ah, ph, 'label')
    
    Use set_plot_area(ah) just before this function.
    
    ah: Axis object
    ph: Plot object
    label: string
    
    """
    axins1 = ah.inset_axes( (0.5, 1.05, .5, .05) ) # relative units
    # ph.set_cmap(plt.cm.YlGnBu_r)
    ph.set_clim(*clims)
    ch = ah.get_figure().colorbar(ph, cax=axins1, orientation='horizontal', shrink=0.5)
    axins1.xaxis.set_ticks_position('top')
    th = axins1.text(-0.1, 0, label, 
                     horizontalalignment='right', 
                     transform=axins1.transAxes) # set later with th.set_text('')
    return ch, axins1, th

#%% Import exp data
dirname = ''
filenames = ['series_Bz_2020-11-30_19-17-54_qtt_scan2DresonatorturboRF.mat',
             'series_Bz_2020-11-30_19-18-13_qtt_scan2DresonatorturboRF.mat',
             'series_Bz_2020-11-30_19-18-33_qtt_scan2DresonatorturboRF.mat', ]
X_OFFSET_PX = 520
Y_OFFSET_PX = 520



#% DQD1
mat_contents = sio.loadmat(os.path.join(dirname, filenames[0]))
x_data = mat_contents['series']['x_bfield'].item()
y_data = mat_contents['series']['y_frequency'].item()
z_data = mat_contents['series']['z_S21mag'].item()
title = mat_contents['series']['id'].item()[0] + '\n' + mat_contents['series']['tag'].item()[0]

plt.close(11)
fh, ah = plt.subplots(num=11)

plt.sca(ah)
# indices[y, x]
ph = plt.pcolormesh(x_data, y_data/1e9, z_data/1e-4, **pcolor_opts)
plt.xlabel(r'Magnetic field, $B_r$ (T)')
plt.ylabel(r'$f_\mathrm{probe}$ (GHz)')
# plt.colorbar(label='S21mag')
set_plot_area(ah)
ch, _, _ = make_colorbar_top(ah, ph, '|S21| (arb. u.)')

# plt.title(title)
xlim = plt.gca().get_xlim()
ylim = plt.gca().get_ylim()
ylim = (6.87575, 6.93275)

ah.xaxis.set_major_locator(plt.MultipleLocator(.005))
# ah.xaxis.set_major_formatter(plt.FormatStrFormatter('%d'))
ah.xaxis.set_minor_locator(plt.MultipleLocator(.001))
# ah.xaxis.set_minor_formatter(plt.FormatStrFormatter('%d'))

x_Br = np.linspace(46e-3, 60e-3, 301)
y_energy = calc_energies(H_system, **mod_param(params, 'Br', x_Br, 'eps2', 100e9))
y_transitions = calc_transitions(y_energy[:,0:6])

# for i in [10, 11, 12]:
#     plt.plot(x_Br, y_transitions[:,i]/1e9, color='tab:orange')

i11 = 12
i12 = 11
i2 = 10
i_split = np.argmin(np.abs(y_transitions[:,i11] - y_transitions[:,i12])) + 0
y_transitions_1 = np.hstack((y_transitions[0:i_split,i11], y_transitions[i_split:,i12]))
y_transitions_2 = y_transitions[:,i2]
plt.plot(x_Br, y_transitions_1/1e9, color='tab:orange', dashes=[6, 6])
plt.plot(x_Br, y_transitions_2/1e9, color='tab:orange', dashes=[6, 6])

plt.gca().set_xlim(xlim)
plt.gca().set_ylim(ylim)
# plt.tight_layout()
plt.show()
move_figure(fh, 0*X_OFFSET_PX, 0)
plt.savefig('Fig_DQD1.pdf')

gs1 = np.min(y_transitions_1-y_transitions_2)/2

# #linecut
# FIG_WIDTH2 = 2-.6 # cm
# FIG_HEIGHT2 = 5.5 # cm
# plt.close(71)
# fh, ah = plt.subplots(num=71, figsize=[FIG_WIDTH2/2.51, FIG_HEIGHT2/2.51])
# idx = np.argmin(np.abs(x_data[0,:]-0.05345))
# # ph = plt.plot(y_data[:,idx]/1e9, np.mean(z_data[:,idx-0:idx+1],axis=1)/1e-4)
# ph = plt.plot(np.mean(z_data[:,idx-0:idx+1],axis=1)/1e-4, y_data[:,idx]/1e9, rasterized=True)
# ah.set_position((0.2/FIG_WIDTH2, 1.0/FIG_HEIGHT2, 1.1/FIG_WIDTH2, 3.5/FIG_HEIGHT2))  # relative units
# plt.gca().set_ylim(ylim)
# plt.xlabel('|S21| (arb. u.)')
# ah.get_yaxis().set_ticklabels([])
# plt.savefig('Fig_DQD1_cut.pdf')



#% DQD2
mat_contents = sio.loadmat(os.path.join(dirname, filenames[1]))
x_data = mat_contents['series']['x_bfield'].item()
y_data = mat_contents['series']['y_frequency'].item()
z_data = mat_contents['series']['z_S21mag'].item()
title = mat_contents['series']['id'].item()[0] + '\n' + mat_contents['series']['tag'].item()[0]

plt.close(12)
fh, ah = plt.subplots(num=12)

plt.sca(ah)
# indices[y, x]
ph = plt.pcolormesh(x_data, y_data/1e9, z_data/1e-4, **pcolor_opts)
plt.xlabel(r'Magnetic field, $B_r$ (T)')
plt.ylabel(r'$f_\mathrm{probe}$ (GHz)')
# plt.colorbar(label='S21mag')
set_plot_area(ah)
ch, _, _ = make_colorbar_top(ah, ph, '|S21| (arb. u.)')

# plt.title(title)
xlim = plt.gca().get_xlim()
ylim = plt.gca().get_ylim()
ylim = (6.87575, 6.93275)

ah.xaxis.set_major_locator(plt.MultipleLocator(.005))
# ah.xaxis.set_major_formatter(plt.FormatStrFormatter('%d'))
ah.xaxis.set_minor_locator(plt.MultipleLocator(.001))
# ah.xaxis.set_minor_formatter(plt.FormatStrFormatter('%d'))

x_Br = np.linspace(46e-3, 60e-3, 301)
y_energy = calc_energies(H_system, **mod_param(params, 'Br', x_Br, 'eps1', 100e9))
y_transitions = calc_transitions(y_energy[:,0:6])

# for i in [10, 11, 12]:
#     plt.plot(x_Br, y_transitions[:,i]/1e9, color='tab:orange')

i11 = 12 # unwind curves, section 1
i12 = 11 # unwind curves, section 2
i2 = 10
i_split = np.argmin(np.abs(y_transitions[:,i11] - y_transitions[:,i12])) + 1
y_transitions_1 = np.hstack((y_transitions[0:i_split,i11], y_transitions[i_split:,i12]))
y_transitions_2 = y_transitions[:,i2]
plt.plot(x_Br, y_transitions_1/1e9, color='tab:orange', dashes=[6, 6])
plt.plot(x_Br, y_transitions_2/1e9, color='tab:orange', dashes=[6, 6])

plt.gca().set_xlim(xlim)
plt.gca().set_ylim(ylim)
# plt.tight_layout()
plt.show()
move_figure(fh, 1*X_OFFSET_PX, 0)
plt.savefig('Fig_DQD2.pdf')

gs2 = np.min(y_transitions_1-y_transitions_2)/2

# #linecut
# plt.close(72)
# fh, ah = plt.subplots(num=72)
# idx = np.argmin(np.abs(x_data[0,:]-0.0536))
# ph = plt.plot(y_data[:,idx]/1e9, np.mean(z_data[:,idx-1:idx+1],axis=1)/1e-4)



#% DQD1+DQD2
mat_contents = sio.loadmat(os.path.join(dirname, filenames[2]))
x_data = mat_contents['series']['x_bfield'].item()
y_data = mat_contents['series']['y_frequency'].item()
z_data = mat_contents['series']['z_S21mag'].item()
title = mat_contents['series']['id'].item()[0] + '\n' + mat_contents['series']['tag'].item()[0]

plt.close(13)
fh, ah = plt.subplots(num=13)

plt.sca(ah)
# indices[y, x]
ph = plt.pcolormesh(x_data, y_data/1e9, z_data/1e-4, **pcolor_opts)
plt.xlabel(r'Magnetic field, $B_r$ (T)')
plt.ylabel(r'$f_\mathrm{probe}$ (GHz)')
# plt.colorbar(label='S21mag')
set_plot_area(ah)
ch, _, _ = make_colorbar_top(ah, ph, '|S21| (arb. u.)')

# plt.title(title)
xlim = plt.gca().get_xlim()
ylim = plt.gca().get_ylim()
ylim = (6.87575, 6.93275)

ah.xaxis.set_major_locator(plt.MultipleLocator(.005))
# ah.xaxis.set_major_formatter(plt.FormatStrFormatter('%d'))
ah.xaxis.set_minor_locator(plt.MultipleLocator(.001))
# ah.xaxis.set_minor_formatter(plt.FormatStrFormatter('%d'))

x_Br = np.linspace(46e-3, 60e-3, 301)
y_energy = calc_energies(H_system, **mod_param(params, 'Br', x_Br))
y_transitions = calc_transitions(y_energy[:,0:6])

for i in [10, 11, 12]:
    plt.plot(x_Br, y_transitions[:,i]/1e9, color='tab:orange', dashes=[6, 6])

plt.gca().set_xlim(xlim)
plt.gca().set_ylim(ylim)
# plt.tight_layout()
plt.show()
move_figure(fh, 2*X_OFFSET_PX, 0)
plt.savefig('Fig_DQD12.pdf')

gs12 = np.min(y_transitions[:,12]-y_transitions[:,10])/2

# #linecut
# plt.close(73)
# fh, ah = plt.subplots(num=73)
# idx = np.argmin(np.abs(x_data[0,:]-0.05340))
# ph = plt.plot(y_data[:,idx]/1e9, np.mean(z_data[:,idx-1:idx+1],axis=1)/1e-4)



#%% Matplotlib rc config 
# plt.rcdefaults() # reset rc defaluts

FONT_SIZE = 7
TITLE_SIZE = 7
LW = 0.75
# set subscript font size
matplotlib.mathtext.SHRINK_FACTOR = 0.8

FIG_WIDTH = 2.5 # cm
FIG_HEIGHT = 2.8 # cm
fig_size_list = [FIG_WIDTH/2.51, FIG_HEIGHT/2.51] # in inches
ax_position_tup = (1.3/FIG_WIDTH, 1.0/FIG_HEIGHT, 1.0/FIG_WIDTH, 1.6/FIG_HEIGHT) # relative units

plt.rcParams['font.sans-serif'] = ['Helvetica Neue',
                                   'Helvetica',
                                   'DejaVu Sans',
                                   'Bitstream Vera Sans',
                                   'Computer Modern Sans Serif',
                                   'Lucida Grande',
                                   'Verdana',
                                   'Geneva',
                                   'Lucid',
                                   'Arial',
                                   'Helvetica',
                                   'Avant Garde',
                                   'sans-serif']

# plt.rcParams['mathtext.fontset'] = 'dejavusans'
# plt.rcParams['mathtext.fontset'] = 'stixsans'
plt.rcParams['mathtext.fontset'] = 'custom'
plt.rcParams['mathtext.it'] = 'Arial:italic' # closest to helvetica that works...
plt.rcParams['mathtext.rm'] = 'Helvetica Neue' # works

plt.rc('font', size=FONT_SIZE)          # controls default text sizes
plt.rc('axes', labelsize=FONT_SIZE)     # fontsize of the x and y labels
plt.rc('axes', titlesize=TITLE_SIZE)    # fontsize of the axes title
plt.rc('xtick', labelsize=FONT_SIZE)    # fontsize of the tick labels
plt.rc('ytick', labelsize=FONT_SIZE)    # fontsize of the tick labels
plt.rc('legend', fontsize=FONT_SIZE, title_fontsize=None)    # legend fontsize
plt.rc('figure', titlesize=TITLE_SIZE)  # fontsize of the figure title

plt.rc('axes', linewidth=LW)
plt.rc('lines', linewidth=LW)

TL = 2.5
plt.rc('xtick.major', size=TL)           # tick length (default 3.5)
plt.rc('xtick.minor', size=TL/1.75)      # tick length (default 2)
plt.rc('ytick.major', size=TL)           # tick length (default 3.5)
plt.rc('ytick.minor', size=TL/1.75)      # tick length (default 2)

pcolor_opts = {'shading': 'nearest',
               'rasterized': True, }  # pass as **pcolor_opts

plt.rcParams['legend.frameon'] = True

plt.rcParams['figure.dpi'] = 129.0 # adjust to match display
plt.rcParams['figure.figsize'] = fig_size_list # in inches

plt.rcParams['savefig.dpi'] = 300
plt.rcParams['savefig.transparent'] = True

plt.rcParams['pdf.fonttype'] = 3 # 42 (TrueType) to edit PDF ext editor. Default 3 is more compact.

def set_plot_area(ah):
    ah.set_position(ax_position_tup)  # relative units

def make_colorbar_top(ah, ph, label='', clims=(None, None)):
    """ Generates a colorbar at the outer top right corner of the plot. 
    
    Clims must be set in this function, otherwise the colorbar resets itself. Ex.:
    ch, _, _ = make_colorbar_top(ah, ph, 'label')
    
    Use set_plot_area(ah) just before this function.
    
    ah: Axis object
    ph: Plot object
    label: string
    
    """
    axins1 = ah.inset_axes( (0.5, 1.05, .5, .05) ) # relative units
    # ph.set_cmap(plt.cm.YlGnBu_r)
    ph.set_clim(*clims)
    ch = ah.get_figure().colorbar(ph, cax=axins1, orientation='horizontal', shrink=0.5)
    axins1.xaxis.set_ticks_position('top')
    th = axins1.text(-0.1, 0, label, 
                     horizontalalignment='right', 
                     transform=axins1.transAxes) # set later with th.set_text('')
    return ch, axins1, th








# def make_colorbar_top(ah, ph, label='', clims=(None, None)):
#     """ Generates a colorbar at the outer top right corner of the plot. 
    
#     Clims must be set in this function, otherwise the colorbar resets itself. Ex.:
#     ch, _, _ = make_colorbar_top(ah, ph, 'label')
    
#     Use set_plot_area(ah) just before this function.
    
#     ah: Axis object
#     ph: Plot object
#     label: string
    
#     """
#     axins1 = ah.inset_axes( (0.5, 1.05, .5, .05) ) # relative units
#     ph.set_clim(*clims)
#     ch = ah.get_figure().colorbar(ph, cax=axins1, orientation='horizontal', shrink=0.5)
#     axins1.xaxis.set_ticks_position('top')
#     th = axins1.text(-0.1, 0, label, 
#                      horizontalalignment='right', 
#                      transform=axins1.transAxes) # set later with th.set_text('')
#     return ch, axins1, th

#%% disp shift plots
# path of directory containing the data, or current working directory
dirpath = os.getcwd()
io = qc.data.io.DiskIO(dirpath)

filenames = ['23-31-41_qtt_scan2DresonatorturboRF',
             '23-32-00_qtt_scan2DresonatorturboRF',
             '23-32-19_qtt_scan2DresonatorturboRF', ]
X_OFFSET_PX = 520
Y_OFFSET_PX = 520
ticks = np.array([-80, 0, 80])


#% DQD1
ds = qc.data.data_set.load_data(os.path.join(dirpath, filenames[0]), io=io) # loads the GNUplot .dat and snapshop.json
ds.gate = ds.RP1 # proxy
x_data = ds.gate.ndarray
y_data = ds.frequency.ndarray
z_data = ds.S21mag.ndarray
# title = ds.metadata['scantime'][0:10] + '/' + os.path.split(ds.location)[1]

plt.close(21)
fh, ah = plt.subplots(num=21)

plt.sca(ah)
# indices[y, x]
ph = plt.pcolormesh((x_data-(-640.2))*15, y_data/1e9, z_data/1e-4, **pcolor_opts)
plt.xlabel(r'$\epsilon_\mathrm{DQD1}$ (µeV)')
plt.ylabel(r'$f_\mathrm{probe}$ (GHz)')
# plt.colorbar(label='S21mag')
set_plot_area(ah)
# ch, _, _ = make_colorbar_top(ah, ph, '|S21| (arb. u.)')

# plt.title(title)
xlim = plt.gca().get_xlim()
ylim = plt.gca().get_ylim()
ylim = (6.900-.00025, 6.920+.00025)
plt.xticks(ticks)

x_eps = np.linspace(-100e9, 100e9, 301)
y_energy = calc_energies(H_system, **mod_param(params, 'Br', 0.048, 'eps1', x_eps, 'eps2', 100e9))
y_transitions = calc_transitions(y_energy[:,0:6])

for i in [12]:
    plt.plot(x_eps*planckh, y_transitions[:,i]/1e9, color='tab:orange', dashes=[6, 6])

plt.gca().set_xlim(xlim)
plt.gca().set_ylim(ylim)
# plt.tight_layout()
plt.show()
move_figure(fh, 0*X_OFFSET_PX, 1*Y_OFFSET_PX)
plt.savefig('Fig_Disp_DQD1.pdf')



#% DQD2
ds = qc.data.data_set.load_data(os.path.join(dirpath, filenames[1]), io=io) # loads the GNUplot .dat and snapshop.json
ds.gate = ds.RP2 # proxy
x_data = ds.gate.ndarray
y_data = ds.frequency.ndarray
z_data = ds.S21mag.ndarray
# title = ds.metadata['scantime'][0:10] + '/' + os.path.split(ds.location)[1]

plt.close(22)
fh, ah = plt.subplots(num=22)

plt.sca(ah)
# indices[y, x]
ph = plt.pcolormesh((x_data-(-789.2))*15, y_data/1e9, z_data/1e-4, **pcolor_opts)
plt.xlabel(r'$\epsilon_\mathrm{DQD2}$ (µeV)')
plt.ylabel(r'$f_\mathrm{probe}$ (GHz)')
# plt.colorbar(label='S21mag')
set_plot_area(ah)
# ch, _, _ = make_colorbar_top(ah, ph, '|S21| (arb. u.)')

# plt.title(title)
xlim = plt.gca().get_xlim()
ylim = plt.gca().get_ylim()
ylim = (6.900-.00025, 6.920+.00025)
plt.xticks(ticks)

x_eps = np.linspace(-100e9, 100e9, 301)
y_energy = calc_energies(H_system, **mod_param(params, 'Br', 0.048, 'eps2', x_eps, 'eps1', 100e9))
y_transitions = calc_transitions(y_energy[:,0:6])

for i in [12]:
    plt.plot(x_eps*planckh, y_transitions[:,i]/1e9, color='tab:orange', dashes=[6, 6])

plt.gca().set_xlim(xlim)
plt.gca().set_ylim(ylim)
# plt.tight_layout()
plt.show()
move_figure(fh, 1*X_OFFSET_PX, 1*Y_OFFSET_PX)
plt.savefig('Fig_Disp_DQD2.pdf')



#% DQD1+DQD2
ds = qc.data.data_set.load_data(os.path.join(dirpath, filenames[2]), io=io) # loads the GNUplot .dat and snapshop.json
ds.gate = ds.RP1 # proxy
x_data = ds.gate.ndarray
y_data = ds.frequency.ndarray
z_data = ds.S21mag.ndarray
# title = ds.metadata['scantime'][0:10] + '/' + os.path.split(ds.location)[1]

plt.close(23)
fh, ah = plt.subplots(num=23)

plt.sca(ah)
# indices[y, x]
ph = plt.pcolormesh((x_data-(-640.2))*15, y_data/1e9, z_data/1e-4, **pcolor_opts)
plt.xlabel(r'$\epsilon_\mathrm{DQD1}$ (µeV)')
plt.ylabel(r'$f_\mathrm{probe}$ (GHz)')
# plt.colorbar(label='S21mag')
set_plot_area(ah)
# ch, _, _ = make_colorbar_top(ah, ph, '|S21| (arb. u.)')

# plt.title(title)
xlim = plt.gca().get_xlim()
ylim = plt.gca().get_ylim()
ylim = (6.900-.00025, 6.920+.00025)
plt.xticks(ticks)

x_eps = np.linspace(-100e9, 100e9, 301)
y_energy = calc_energies(H_system, **mod_param(params, 'Br', 0.048, 'eps1', x_eps, 'eps2', 0))
y_transitions = calc_transitions(y_energy[:,0:6])

for i in [12]:
    plt.plot(x_eps*planckh, y_transitions[:,i]/1e9, color='tab:orange', dashes=[6, 6])

plt.gca().set_xlim(xlim)
plt.gca().set_ylim(ylim)
# plt.tight_layout()
plt.show()
move_figure(fh, 2*X_OFFSET_PX, 1*Y_OFFSET_PX)
plt.savefig('Fig_Disp_DQD12.pdf')
